import logger from '../../../utils/logger'
import utils from '../../../utils/utils'
import { Request, SubmoduleData, User } from '../../../types/basicTypes'

const idStatFile = 'stats/idstats'
const idvStatFile = 'stats/idvstats'

function mergeObjSum(a, b) {
  const res = { ...b }
  Object.keys(a).forEach((key) => {
    if (res[key]) {
      res[key] += a[key]
    } else {
      res[key] = a[key]
    }
  })

  return res
}

function setup(data: SubmoduleData): void {
  const { app /* userDB, url, publicdirs, moduleSpecificData */ } = data

  app.get('/ranklist', (req: Request, res: any) => {
    logger.LogReq(req)
    let result
    const querySince: any = req.query.since
    const user: User = req.session.user

    if (!querySince) {
      result = utils.ReadJSON(idStatFile)
    } else {
      try {
        const since = new Date(querySince)
        if (!(since instanceof Date) || isNaN(since.getTime())) {
          throw new Error('Not a date')
        }
        const data = utils.ReadJSON(idvStatFile)
        result = {}

        Object.keys(data).forEach((key) => {
          const dailyStat = data[key]

          if (new Date(key) > since) {
            Object.keys(dailyStat).forEach((userId) => {
              const userStat = dailyStat[userId]
              const uidRes = result[userId]

              if (!uidRes) {
                result[userId] = userStat
              } else {
                result[userId] = {
                  count: uidRes.count + userStat.count,
                  newQuestions: uidRes.newQuestions + userStat.newQuestions,
                  allQuestions: uidRes.allQuestions + userStat.allQuestions,
                  subjs: mergeObjSum(uidRes.subjs, userStat.subjs),
                }
              }
            })
          }
        })
      } catch (err) {
        res.json({
          msg: 'invalid date format, or other error occured',
        })
      }
    }

    const list = []
    const sum = {
      count: 0,
      newQuestions: 0,
      allQuestions: 0,
    }
    Object.keys(result).forEach((key) => {
      list.push({
        userId: parseInt(key),
        ...result[key],
      })

      sum.count = sum.count + result[key].count
      sum.newQuestions = sum.newQuestions + result[key].newQuestions
      sum.allQuestions = sum.allQuestions + result[key].allQuestions
    })

    if (list.length === 0) {
      res.json({
        msg: 'There are no users in the stats db :c',
      })
      return
    }

    res.json({
      since: querySince,
      sum: sum,
      list: list,
      selfuserId: user.id,
    })
  })
}

export default {
  setup: setup,
}
